@@ -4,11 +4,11 @@
  *	  interface routines for the postgres GiST index access method.
  *
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/access/gist/gist.c,v 1.88.2.1 2002/05/28 15:22:44 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/access/gist/gist.c,v 1.105 2003/08/04 02:39:56 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -87,12 +87,10 @@ static OffsetNumber gistwritebuffer(Relation r,
 				Page page,
 				IndexTuple *itup,
 				int len,
-				OffsetNumber off,
-				GISTSTATE *giststate);
+				OffsetNumber off);
 static int gistnospace(Page page,
 			IndexTuple *itvec, int len);
-static IndexTuple *gistreadbuffer(Relation r,
-			   Buffer buffer, int *len);
+static IndexTuple *gistreadbuffer(Buffer buffer, int *len);
 static IndexTuple *gistjoinvector(
 			   IndexTuple *itvec, int *len,
 			   IndexTuple *additvec, int addlen);
@@ -117,7 +115,7 @@ static IndexTuple *gistSplit(Relation r,
 		  int *len,
 		  GISTSTATE *giststate,
 		  InsertIndexResult *res);
-static void gistnewroot(GISTSTATE *giststate, Relation r,
+static void gistnewroot(Relation r,
 			IndexTuple *itup, int len);
 static void GISTInitBuffer(Buffer b, uint32 f);
 static OffsetNumber gistchoose(Relation r, Page p,
@@ -170,7 +168,7 @@ gistbuild(PG_FUNCTION_ARGS)
 	 * that's not the case, big trouble's what we have.
 	 */
 	if (RelationGetNumberOfBlocks(index) != 0)
-		elog(ERROR, "%s already contains data",
+		elog(ERROR, "index \"%s\" already contains data",
 			 RelationGetRelationName(index));
 
 	/* initialize the root page */
@@ -299,6 +297,7 @@ gistinsert(PG_FUNCTION_ARGS)
 
 #ifdef NOT_USED
 	Relation	heapRel = (Relation) PG_GETARG_POINTER(4);
+	bool		checkUnique = PG_GETARG_BOOL(5);
 #endif
 	InsertIndexResult res;
 	IndexTuple	itup;
@@ -359,11 +358,11 @@ gistinsert(PG_FUNCTION_ARGS)
 
 #ifdef GIST_PAGEADDITEM
 /*
-** Take a compressed entry, and install it on a page.  Since we now know
-** where the entry will live, we decompress it and recompress it using
-** that knowledge (some compression routines may want to fish around
-** on the page, for example, or do something special for leaf nodes.)
-*/
+ * Take a compressed entry, and install it on a page.  Since we now know
+ * where the entry will live, we decompress it and recompress it using
+ * that knowledge (some compression routines may want to fish around
+ * on the page, for example, or do something special for leaf nodes.)
+ */
 static OffsetNumber
 gistPageAddItem(GISTSTATE *giststate,
 				Relation r,
@@ -397,7 +396,7 @@ gistPageAddItem(GISTSTATE *giststate,
 	retval = PageAddItem(page, (Item) *newtup, IndexTupleSize(*newtup),
 						 offsetNumber, flags);
 	if (retval == InvalidOffsetNumber)
-		elog(ERROR, "gist: failed to add index item to %s",
+		elog(ERROR, "failed to add index item to \"%s\"",
 			 RelationGetRelationName(r));
 	/* be tidy */
 	if (DatumGetPointer(tmpcentry.key) != NULL &&
@@ -425,7 +424,7 @@ gistdoinsert(Relation r,
 
 	ret = gistlayerinsert(r, GISTP_ROOT, &instup, &len, res, giststate);
 	if (ret & SPLITED)
-		gistnewroot(giststate, r, instup, len);
+		gistnewroot(r, instup, len);
 
 	for (i = 0; i < len; i++)
 		pfree(instup[i]);
@@ -452,7 +451,7 @@ gistlayerinsert(Relation r, BlockNumber blkno,
 	if (!(opaque->flags & F_LEAF))
 	{
 		/* internal page, so we must walk on tree */
-		/* len IS equial 1 */
+		/* len IS equal 1 */
 		ItemId		iid;
 		BlockNumber nblkno;
 		ItemPointerData oldtid;
@@ -466,7 +465,7 @@ gistlayerinsert(Relation r, BlockNumber blkno,
 		/*
 		 * After this call: 1. if child page was splited, then itup
 		 * contains keys for each page 2. if  child page wasn't splited,
-		 * then itup contains additional for adjustement of current key
+		 * then itup contains additional for adjustment of current key
 		 */
 		ret = gistlayerinsert(r, nblkno, itup, len, res, giststate);
 
@@ -496,13 +495,13 @@ gistlayerinsert(Relation r, BlockNumber blkno,
 		/* key is modified, so old version must be deleted */
 		ItemPointerSet(&oldtid, blkno, child);
 		gistdelete(r, &oldtid);
-	
+
 		/*
-		 * if child was splitted, new key for child will be inserted
-		 * in the end list of child, so we must say to any scans
-		 * that page is changed beginning from 'child' offset
+		 * if child was splitted, new key for child will be inserted in
+		 * the end list of child, so we must say to any scans that page is
+		 * changed beginning from 'child' offset
 		 */
-		if ( ret & SPLITED )
+		if (ret & SPLITED)
 			gistadjscans(r, GISTOP_SPLIT, blkno, child);
 	}
 
@@ -517,7 +516,7 @@ gistlayerinsert(Relation r, BlockNumber blkno,
 					oldlen;
 
 		ret |= SPLITED;
-		itvec = gistreadbuffer(r, buffer, &tlen);
+		itvec = gistreadbuffer(buffer, &tlen);
 		itvec = gistjoinvector(itvec, &tlen, (*itup), *len);
 		oldlen = *len;
 		newitup = gistSplit(r, buffer, itvec, &tlen, giststate,
@@ -542,7 +541,7 @@ gistlayerinsert(Relation r, BlockNumber blkno,
 			FirstOffsetNumber
 			:
 			OffsetNumberNext(PageGetMaxOffsetNumber(page));
-		l = gistwritebuffer(r, page, (*itup), *len, off, giststate);
+		l = gistwritebuffer(r, page, (*itup), *len, off);
 		WriteBuffer(buffer);
 
 		/*
@@ -552,7 +551,7 @@ gistlayerinsert(Relation r, BlockNumber blkno,
 			ItemPointerSet(&((*res)->pointerData), blkno, l);
 
 		if (*len > 1)
-		{						/* previos insert ret & SPLITED != 0 */
+		{						/* previous insert ret & SPLITED != 0 */
 			int			i;
 
 			/*
@@ -578,7 +577,7 @@ gistlayerinsert(Relation r, BlockNumber blkno,
  */
 static OffsetNumber
 gistwritebuffer(Relation r, Page page, IndexTuple *itup,
-				int len, OffsetNumber off, GISTSTATE *giststate)
+				int len, OffsetNumber off)
 {
 	OffsetNumber l = InvalidOffsetNumber;
 	int			i;
@@ -604,7 +603,7 @@ gistwritebuffer(Relation r, Page page, IndexTuple *itup,
 		l = PageAddItem(page, (Item) itup[i], IndexTupleSize(itup[i]),
 						off, LP_USED);
 		if (l == InvalidOffsetNumber)
-			elog(ERROR, "gist: failed to add index item to %s",
+			elog(ERROR, "failed to add index item to \"%s\"",
 				 RelationGetRelationName(r));
 #endif
 	}
@@ -617,7 +616,7 @@ gistwritebuffer(Relation r, Page page, IndexTuple *itup,
 static int
 gistnospace(Page page, IndexTuple *itvec, int len)
 {
-	int			size = 0;
+	unsigned int size = 0;
 	int			i;
 
 	for (i = 0; i < len; i++)
@@ -630,7 +629,7 @@ gistnospace(Page page, IndexTuple *itvec, int len)
  * Read buffer into itup vector
  */
 static IndexTuple *
-gistreadbuffer(Relation r, Buffer buffer, int *len /* out */ )
+gistreadbuffer(Buffer buffer, int *len /* out */ )
 {
 	OffsetNumber i,
 				maxoff;
@@ -681,7 +680,7 @@ gistunion(Relation r, IndexTuple *itvec, int len, GISTSTATE *giststate)
 
 	needfree = (bool *) palloc(((len == 1) ? 2 : len) * sizeof(bool));
 	/* workaround for 64-bit: ensure GISTENTRY array is maxaligned */
-	storage = (char*)palloc( ((len == 1) ? 2 : len) * sizeof(GISTENTRY) + MAXALIGN(VARHDRSZ));
+	storage = (char *) palloc(((len == 1) ? 2 : len) * sizeof(GISTENTRY) + MAXALIGN(VARHDRSZ));
 	evec = (bytea *) (storage + MAXALIGN(VARHDRSZ) - VARHDRSZ);
 
 	for (j = 0; j < r->rd_att->natts; j++)
@@ -788,7 +787,7 @@ gistgetadjusted(Relation r, IndexTuple oldtup, IndexTuple addtup, GISTSTATE *gis
 	int			j;
 
 	/* workaround for 64-bit: ensure GISTENTRY array is maxaligned */
-	storage = (char*) palloc( 2 * sizeof(GISTENTRY) + MAXALIGN(VARHDRSZ));
+	storage = (char *) palloc(2 * sizeof(GISTENTRY) + MAXALIGN(VARHDRSZ));
 	evec = (bytea *) (storage + MAXALIGN(VARHDRSZ) - VARHDRSZ);
 	VARATT_SIZEP(evec) = 2 * sizeof(GISTENTRY) + VARHDRSZ;
 	ev0p = &((GISTENTRY *) VARDATA(evec))[0];
@@ -913,7 +912,7 @@ gistunionsubkey(Relation r, GISTSTATE *giststate, IndexTuple *itvec, GIST_SPLITV
 
 		needfree = (bool *) palloc(((len == 1) ? 2 : len) * sizeof(bool));
 		/* workaround for 64-bit: ensure GISTENTRY array is maxaligned */
-		storage = (char*)palloc( ((len == 1) ? 2 : len) * sizeof(GISTENTRY) + MAXALIGN(VARHDRSZ));
+		storage = (char *) palloc(((len == 1) ? 2 : len) * sizeof(GISTENTRY) + MAXALIGN(VARHDRSZ));
 		evec = (bytea *) (storage + MAXALIGN(VARHDRSZ) - VARHDRSZ);
 
 		for (j = 1; j < r->rd_att->natts; j++)
@@ -1100,7 +1099,7 @@ gistadjsubkey(Relation r,
 	v->spl_nright = curlen;
 
 	/* workaround for 64-bit: ensure GISTENTRY array is maxaligned */
-	storage = (char*)palloc( 2 * sizeof(GISTENTRY) + MAXALIGN(VARHDRSZ));
+	storage = (char *) palloc(2 * sizeof(GISTENTRY) + MAXALIGN(VARHDRSZ));
 	evec = (bytea *) (storage + MAXALIGN(VARHDRSZ) - VARHDRSZ);
 	VARATT_SIZEP(evec) = 2 * sizeof(GISTENTRY) + VARHDRSZ;
 	ev0p = &((GISTENTRY *) VARDATA(evec))[0];
@@ -1278,7 +1277,7 @@ gistSplit(Relation r,
 	/* workaround for 64-bit: ensure GISTENTRY array is maxaligned */
 	storage = palloc(MAXALIGN(VARHDRSZ) + (*len + 1) * sizeof(GISTENTRY));
 	entryvec = (bytea *) (storage + MAXALIGN(VARHDRSZ) - VARHDRSZ);
-	decompvec = (bool *) palloc( (*len + 1) * sizeof(bool));
+	decompvec = (bool *) palloc((*len + 1) * sizeof(bool));
 	VARATT_SIZEP(entryvec) = (*len + 1) * sizeof(GISTENTRY) + VARHDRSZ;
 	for (i = 1; i <= *len; i++)
 	{
@@ -1317,10 +1316,8 @@ gistSplit(Relation r,
 	 */
 	if (r->rd_att->natts > 1)
 	{
-		v.spl_idgrp = (int *) palloc(sizeof(int) * (*len + 1));
-		MemSet((void *) v.spl_idgrp, 0, sizeof(int) * (*len + 1));
-		v.spl_grpflag = (char *) palloc(sizeof(char) * (*len + 1));
-		MemSet((void *) v.spl_grpflag, 0, sizeof(char) * (*len + 1));
+		v.spl_idgrp = (int *) palloc0(sizeof(int) * (*len + 1));
+		v.spl_grpflag = (char *) palloc0(sizeof(char) * (*len + 1));
 		v.spl_ngrp = (int *) palloc(sizeof(int) * (*len + 1));
 
 		MaxGrpId = gistfindgroup(giststate, (GISTENTRY *) VARDATA(entryvec), &v);
@@ -1373,7 +1370,7 @@ gistSplit(Relation r,
 	{
 		OffsetNumber l;
 
-		l = gistwritebuffer(r, right, rvectup, v.spl_nright, FirstOffsetNumber, giststate);
+		l = gistwritebuffer(r, right, rvectup, v.spl_nright, FirstOffsetNumber);
 		WriteBuffer(rightbuf);
 
 		if (res)
@@ -1406,7 +1403,7 @@ gistSplit(Relation r,
 	{
 		OffsetNumber l;
 
-		l = gistwritebuffer(r, left, lvectup, v.spl_nleft, FirstOffsetNumber, giststate);
+		l = gistwritebuffer(r, left, lvectup, v.spl_nleft, FirstOffsetNumber);
 		if (BufferGetBlockNumber(buffer) != GISTP_ROOT)
 			PageRestoreTempPage(left, p);
 
@@ -1432,7 +1429,7 @@ gistSplit(Relation r,
 }
 
 static void
-gistnewroot(GISTSTATE *giststate, Relation r, IndexTuple *itup, int len)
+gistnewroot(Relation r, IndexTuple *itup, int len)
 {
 	Buffer		b;
 	Page		p;
@@ -1441,7 +1438,7 @@ gistnewroot(GISTSTATE *giststate, Relation r, IndexTuple *itup, int len)
 	GISTInitBuffer(b, 0);
 	p = BufferGetPage(b);
 
-	gistwritebuffer(r, p, itup, len, FirstOffsetNumber, giststate);
+	gistwritebuffer(r, p, itup, len, FirstOffsetNumber);
 	WriteBuffer(b);
 }
 
@@ -1596,7 +1593,6 @@ gistbulkdelete(PG_FUNCTION_ARGS)
 	BlockNumber num_pages;
 	double		tuples_removed;
 	double		num_index_tuples;
-	RetrieveIndexResult res;
 	IndexScanDesc iscan;
 
 	tuples_removed = 0;
@@ -1613,23 +1609,22 @@ gistbulkdelete(PG_FUNCTION_ARGS)
 	 */
 
 	/* walk through the entire index */
-	iscan = index_beginscan(rel, false, 0, (ScanKey) NULL);
+	iscan = index_beginscan(NULL, rel, SnapshotAny, 0, (ScanKey) NULL);
+	/* including killed tuples */
+	iscan->ignore_killed_tuples = false;
 
-	while ((res = index_getnext(iscan, ForwardScanDirection))
-		   != (RetrieveIndexResult) NULL)
+	while (index_getnext_indexitem(iscan, ForwardScanDirection))
 	{
-		ItemPointer heapptr = &res->heap_iptr;
-
-		if (callback(heapptr, callback_state))
+		if (callback(&iscan->xs_ctup.t_self, callback_state))
 		{
-			ItemPointer indexptr = &res->index_iptr;
+			ItemPointerData indextup = iscan->currentItemData;
 			BlockNumber blkno;
 			OffsetNumber offnum;
 			Buffer		buf;
 			Page		page;
 
-			blkno = ItemPointerGetBlockNumber(indexptr);
-			offnum = ItemPointerGetOffsetNumber(indexptr);
+			blkno = ItemPointerGetBlockNumber(&indextup);
+			offnum = ItemPointerGetOffsetNumber(&indextup);
 
 			/* adjust any scans that will be affected by this deletion */
 			gistadjscans(rel, GISTOP_DEL, blkno, offnum);
@@ -1646,8 +1641,6 @@ gistbulkdelete(PG_FUNCTION_ARGS)
 		}
 		else
 			num_index_tuples += 1;
-
-		pfree(res);
 	}
 
 	index_endscan(iscan);
@@ -1655,10 +1648,10 @@ gistbulkdelete(PG_FUNCTION_ARGS)
 	/* return statistics */
 	num_pages = RelationGetNumberOfBlocks(rel);
 
-	result = (IndexBulkDeleteResult *) palloc(sizeof(IndexBulkDeleteResult));
+	result = (IndexBulkDeleteResult *) palloc0(sizeof(IndexBulkDeleteResult));
 	result->num_pages = num_pages;
-	result->tuples_removed = tuples_removed;
 	result->num_index_tuples = num_index_tuples;
+	result->tuples_removed = tuples_removed;
 
 	PG_RETURN_POINTER(result);
 }
@@ -1670,7 +1663,7 @@ initGISTstate(GISTSTATE *giststate, Relation index)
 	int			i;
 
 	if (index->rd_att->natts > INDEX_MAX_KEYS)
-		elog(ERROR, "initGISTstate: numberOfAttributes %d > %d",
+		elog(ERROR, "numberOfAttributes %d > %d",
 			 index->rd_att->natts, INDEX_MAX_KEYS);
 
 	giststate->tupdesc = index->rd_att;
@@ -1932,7 +1925,7 @@ gist_dumptree(Relation r, int level, BlockNumber blk, OffsetNumber coff)
 
 	maxoff = PageGetMaxOffsetNumber(page);
 
-	elog(NOTICE, "%sPage: %d %s blk: %d maxoff: %d free: %d", pred,
+	elog(DEBUG4, "%sPage: %d %s blk: %d maxoff: %d free: %d", pred,
 		 coff, (opaque->flags & F_LEAF) ? "LEAF" : "INTE", (int) blk,
 		 (int) maxoff, PageGetFreeSpace(page));
 
@@ -1942,7 +1935,7 @@ gist_dumptree(Relation r, int level, BlockNumber blk, OffsetNumber coff)
 		which = (IndexTuple) PageGetItem(page, iid);
 		cblk = ItemPointerGetBlockNumber(&(which->t_tid));
 #ifdef PRINTTUPLE
-		elog(NOTICE, "%s  Tuple. blk: %d size: %d", pred, (int) cblk,
+		elog(DEBUG4, "%s  Tuple. blk: %d size: %d", pred, (int) cblk,
 			 IndexTupleSize(which));
 #endif
 
@@ -1957,13 +1950,13 @@ gist_dumptree(Relation r, int level, BlockNumber blk, OffsetNumber coff)
 void
 gist_redo(XLogRecPtr lsn, XLogRecord *record)
 {
-	elog(STOP, "gist_redo: unimplemented");
+	elog(PANIC, "gist_redo: unimplemented");
 }
 
 void
 gist_undo(XLogRecPtr lsn, XLogRecord *record)
 {
-	elog(STOP, "gist_undo: unimplemented");
+	elog(PANIC, "gist_undo: unimplemented");
 }
 
 void
