@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/executor/execUtils.c,v 1.106 2003/10/01 21:30:52 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/executor/execUtils.c,v 1.106.2.1 2003/12/18 20:21:53 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -18,6 +18,7 @@
  *		FreeExecutorState
  *		CreateExprContext
  *		FreeExprContext
+ *		ReScanExprContext
  *
  *		ExecAssignExprContext	Common code for plan node init routines.
  *		ExecAssignResultType
@@ -352,6 +353,24 @@ FreeExprContext(ExprContext *econtext)
 	pfree(econtext);
 }
 
+/*
+ * ReScanExprContext
+ *
+ *		Reset an expression context in preparation for a rescan of its
+ *		plan node.  This requires calling any registered shutdown callbacks,
+ *		since any partially complete set-returning-functions must be canceled.
+ *
+ * Note we make no assumption about the caller's memory context.
+ */
+void
+ReScanExprContext(ExprContext *econtext)
+{
+	/* Call any registered callbacks */
+	ShutdownExprContext(econtext);
+	/* And clean up the memory used */
+	MemoryContextReset(econtext->ecxt_per_tuple_memory);
+}
+
 /*
  * Build a per-output-tuple ExprContext for an EState.
  *
